//
//  PollingManager.swift
//  MesonWrap
//
//  Created by Iti Agrawal on 09/02/21.
//  Copyright © 2021 InMobi. All rights reserved.
//

import Foundation

class PollingManager {
    
    var eventHandler: (() -> Void)?
    private let pollingInterval: TimeInterval
    
    private lazy var timer: DispatchSourceTimer = {
        let sourceTimer = DispatchSource.makeTimerSource()
        sourceTimer.schedule(deadline: .now() + self.pollingInterval, repeating: self.pollingInterval)
        sourceTimer.setEventHandler(handler: { [weak self] in
            self?.eventHandler?()
        })
        return sourceTimer
    }()
    
    private enum State {
        case suspended
        case resumed
    }
    
    private var state: State = .suspended
    
    init(pollingInterval: TimeInterval) {
        self.pollingInterval = pollingInterval
    }
    
    func resume() {
        if self.state == .resumed {
            return
        }
        self.state = .resumed
        self.timer.resume()
    }
    
    func suspend() {
        if self.state == .suspended {
            return
        }
        self.state = .suspended
        self.timer.suspend()
    }
    
    deinit {
        timer.setEventHandler {}
        timer.cancel()
        /*
         If the timer is suspended, calling cancel without resuming
         triggers a crash. This is documented here https://forums.developer.apple.com/thread/15902
         */
        resume()
        eventHandler = nil
    }
}
